module cache_controller(
	 input clk,
    input [6:0] tag, //tag field
    input [5:0] index,//cache line
    input wr_en, //write/read signal (1,0)
    input flush, //reset cache - set all valid bits to zero
    input ready, //data block ready sig from memory
    output stall, //stall the processor (1,0)
    output wr_en_mem,//write/read signal to memory (1,0)
    output update_cache,//update data array
	 output reg cache_hit //indicate cache hit/loss (1,0)
    );
	 
integer i; //used to flush and initialize tag and validity fields

reg [6:0] tag_field [63:0]; 
reg validity_field [63:0];


initial begin
	for (i=0;i<64;i=i+1) begin
		validity_field[i]=1'b0;
		tag_field[i]=1'b0;
	end
end

assign update_cache=ready; 
assign stall=~(cache_hit|wr_en);
assign wr_en_mem=wr_en; 

always @(posedge clk) begin 

	if (validity_field[index]) cache_hit=(tag_field[index]==tag); 
	else cache_hit=0;

	if (~wr_en) begin
		if (ready) begin 
			validity_field[index]=1;
			tag_field[index]=tag;
		end
	end
	
	if (flush) begin 
		for (i=0;i<64;i=i+1) begin
			validity_field[i]=1'b0;
		end
	end
	
end

endmodule

