`timescale 1ns / 1ps

module inst_cache_tb;
  reg clk;

  wire [31:0] data_in;
  wire [31:0] data_out;
  wire [31:0] rd_addr;
  wire        rd_en;
  wire [31:0] wr_addr;
  wire        wr_en;
  reg  [31:0] i_cache_addr = 0;
  reg  [31:0] i_cache_din = 0;
  reg         i_cache_rden = 0;
  reg         i_cache_wren = 0;
  wire        i_cache_hit_miss;
  wire [31:0] i_cache_q;

  inst_cache DUT_CACHE (
    .clock(clk),
    .address(i_cache_addr),
    .din(i_cache_din),
    .rden(i_cache_rden),
    .wren(i_cache_wren),
    .hit_miss(i_cache_hit_miss),
    .q(i_cache_q),
    .mdout(data_in),
    .mrdaddress(rd_addr),
    .mrden(rd_en),
    .mwraddress(wr_addr),
    .mwren(wr_en),
    .mq(data_out)
  );

  defparam DUT_CACHE.SIZE = 16*1024*8;
  defparam DUT_CACHE.NWAYS = 8;
  defparam DUT_CACHE.NSETS = 2048;
  defparam DUT_CACHE.BLOCK_SIZE = 64;
  defparam DUT_CACHE.WIDTH = 32;
  defparam DUT_CACHE.MWIDTH = 64;

  main_memory DUT_MEM (
    .clock(clk),
    .data(data_in),
    .rdaddress(rd_addr),
    .rden(rd_en),
    .wraddress(wr_addr),
    .wren(wr_en),
    .q(data_out)
  );

  defparam DUT_MEM.WIDTH = 32;
  defparam DUT_MEM.DEPTH = 128;
  defparam DUT_MEM.FILE = "main_mem_text.txt";


  always
    begin
 # 100; // This should be a miss

      i_cache_addr <= 32'h00000004;
      i_cache_din <= 0;
      i_cache_rden <= 1;
      i_cache_wren <= 0;

      # 100; // This should be a miss

      i_cache_addr <= 32'h10000004;
      i_cache_din <= 0;
      i_cache_rden <= 1;
      i_cache_wren <= 0;

      # 100; // This should be a hit

      i_cache_addr <= 32'h00000005;
      i_cache_din <= 32'hBADDBEEF;
      i_cache_rden <= 1;
      i_cache_wren <= 0;

      # 100; // This should be a hit

      i_cache_addr <= 32'h10000006;
      i_cache_din <= 32'h00000000;
      i_cache_rden <= 1;
      i_cache_wren <= 0;

      # 100; // This should be a hit

      i_cache_addr <= 32'h00000007;
      i_cache_din <= 32'hAAAAAAAA;
      i_cache_rden <= 1;
      i_cache_wren <= 0;

      #50;
 $finish;
    end

  initial
  begin
    $monitor("time=%4d | addr=%10d | hm=%b | q=%08x | way1=%08h | way2=%08h | way3=%08h | way4=%08h | way5=%08h | way6=%08h | way7=%08h | way8=%08h | mem1=%08h | lru1=%d | lru2=%d | lru3=%d | lru4=%d | lru5=%d | lru6=%d | lru7=%d | lru8=%d" ,$time,i_cache_addr,i_cache_hit_miss,DUT_CACHE.q,DUT_CACHE.mem1[1],DUT_CACHE.mem2[1],DUT_CACHE.mem3[1],DUT_CACHE.mem4[1],DUT_CACHE.mem5[1],DUT_CACHE.mem6[1],DUT_CACHE.mem7[1],DUT_CACHE.mem8[1],DUT_MEM.mem[1],DUT_CACHE.lru1[1],DUT_CACHE.lru2[1],DUT_CACHE.lru3[1],DUT_CACHE.lru4[1],DUT_CACHE.lru5[1],DUT_CACHE.lru6[1],DUT_CACHE.lru7[1],DUT_CACHE.lru8[1]);
  end
 
  always
    begin
      clk = 1'b1;
      #5;
      clk = 1'b0;
      #5;
    end
   
endmodule
