module Alu (Instruction, Imm_Extension_E, Aluout, a,b, ZeroE,PC, PCTarget_E);
//Declaration of Inputs
input [31:0] Instruction;
input signed [31:0] Imm_Extension_E;
input [ 31:0] a,b;
input signed [31:0] PC;
//Declarations of Outputs
output reg signed [31:0] Aluout;
output reg ZeroE; 
output reg  signed [31:0] PCTarget_E;
//Declaring wires 
wire signed [31:0]A = a;
wire signed [31:0]B = b;
//Declaration of Parameters
// Instruction Types
parameter [7:0] r_to_r   = 7'b0110011,
	        r_to_imm = 7'b0010011,
	        ld       = 7'b0000011,
	        store    = 7'b0100011,
	        upperimm = 7'b0110111,
	        branch   = 7'b1100011;
                //CSR  = 7'b1110011;
					 
// REGISTER TO REGISTER TYPE
parameter [9:0] add  = 10'b0000000_000, //Addition
		sub  = 10'b0100000_000, //Subtraction
                slt  = 10'b0000000_010, //Set if less than
	        sll  = 10'b0000000_001, //Shift left logic
	        sltu = 10'b0000000_011, //Set if less than Unsigned
		Xor  = 10'b0000000_100, //Bitwise Xor
		srl  = 10'b0000000_101, //Shift right logic
		sra  = 10'b0100000_101, //Shift right arithmatic
		Or   = 10'b0000000_110, //Bitwise Or
		And  = 10'b0000000_111; //Bitwise And

// REGISTER TO IMMEDIATE TYPE
parameter [2:0] addi  = 3'b000, //Addition 
	        slli  = 3'b001, //Set if less than
                slti  = 3'b010, //Shift left logic
	        sltui = 3'b011, //Set if less than Unsigned
        	Xori  = 3'b100, //Bitwise Xor
        	srli  = 3'b101, //Shift right logic
	        //srai  = 3'b101, //Shift right arithmatic
	        Ori   = 3'b110, //Bitwise Or
	        andi  = 3'b111; //Bitwise And

// LOAD TYPE
parameter [2:0] LB  = 3'b000, //Load Byte
		LH  = 3'b001, //Load Halfword
		LW  = 3'b010, //Load Word
		LBU = 3'b100, //Load Byte Unsigned
		LHU = 3'B101; //Load Halfword Unsigned

// STORE TYPE
parameter [2:0] SB = 3'b000, // STORE BYTE 
		SH = 3'b001, // STORE HALF WORD
		SW = 3'b010; // STORE WORD

// UPPER IMMEDIATE TYPE
parameter [6:0] LUI   = 7'b0110111, // LOAD UPPER IMMEDIATE 
		AUIPC = 7'b0010111; //ADD UPPER IMMEDIATE
					 
// BRANCH TYPE
parameter [2:0] BEQ  = 3'b000, // BRANCH EQUAL
	     	BNE  = 3'b001, // BRANCH NOT EQUAL
		BLT  = 3'b100, // BRANCH LESS THAN
		BGE  = 3'b101, // BRANCH GREATER THAN
		BLTU = 3'b110, // BRANCH LESS THAN UNSINGNED  
        	BGEU = 3'b111; // BRANCH GREATER THAN UNSINGED

// JUMP TYPE
parameter [6:0] JAL  = 7'b1101111, //JUMP AND LINK
		JALR = 7'b1100111; //JUMP AND LINK REGISTER
/*		
// CSR Type
parameter [2:0] CSRRW = 3'b001,
                CSRRS = 3'b010,
                CSRRC = 3'b011,
                CSRRWI = 3'b101,
                CSRRSI = 3'b110,
                CSRRCI = 3'b111;
 */               
always@(*)
 begin
  case(Instruction [6:0] )
   r_to_r : //Register to Register
    begin
     ZeroE      = 1'b0;
     PCTarget_E = 32'b0;
     Aluout     = 32'b0;
      case({Instruction [31:25] ,Instruction[14:12]}) //Funct7,Funct3 Values
	add : Aluout = A + B;
	sub : Aluout = A - B;
	sll : Aluout = A << B[4:0];
	slt : begin
	       if(A < B)
	        Aluout = 1;
	       else
	        Aluout = 0; 
	      end
	sltu: begin
  	       if(a < b)
		Aluout = 1;
	       else
		Aluout = 0; 
	      end
	Xor : Aluout = A ^ B;
	srl : Aluout = A >> B[4:0];
	sra : Aluout = A >>> B[4:0];
	Or  : Aluout = A || B;
	And : Aluout = A && B;
	default :begin
	          Aluout     = 32'b0;
	          ZeroE      = 1'b0;
	          PCTarget_E = 32'b0;
	 	 end
      endcase
    end
		
   r_to_imm:
    begin
     ZeroE      = 1'b0;
     PCTarget_E = 32'b0;
     Aluout     = 32'b0;
      case(Instruction[14:12] ) //Funct3 values
	addi : Aluout = A + B;
	slli : Aluout = A << b;
	slti : begin
                if(A < B)
  		             Aluout = 1;
	           	else
		             Aluout = 0; 
	       end
	sltui: begin
		if(a < b)
		 Aluout = 1;
		else
		 Aluout = 0; 
	       end
	Xori : Aluout = A ^ B;
	srli : begin
		if(B[10]  == 1)
		 Aluout = A >>> b;
		else
		 Aluout = A >> b;
	       end
	Ori  : Aluout = A | B;
	andi : Aluout = A & B;
	default :begin
		  Aluout     = 32'b0;
		  ZeroE      = 1'b0;
		  PCTarget_E = 32'b0;
		 end
      endcase
    end
   ld:
    begin
     ZeroE      = 1'b0;
     PCTarget_E = 32'b0;
     Aluout     = A + B;
    end
   store:
    begin
     Aluout     = A + B;
     ZeroE      = 1'b0;
     PCTarget_E = 32'b0;
    end
   branch:
    begin
     ZeroE      = 1'b0; 
     PCTarget_E = 32'b0;
     Aluout     = 32'b0;
      case(Instruction [14:12])
	BEQ : begin
	       if(A == B)
		begin
		 ZeroE      = 1'b1;
		 PCTarget_E = PC + Imm_Extension_E;
		end
	       else
		begin
		 ZeroE      = 1'b0; 
		 PCTarget_E = 32'b0;
		end
	      end
	BNE : begin
	       if(A != B)
		begin
		 ZeroE      = 1'b1;
		 PCTarget_E = PC + Imm_Extension_E;
		end
	       else 
		begin
		 ZeroE      = 1'b0; 
		 PCTarget_E = 32'b0;
		end
	      end
	BLT : begin
	       if(A < B)
		begin
		 ZeroE      = 1'b1;
		 PCTarget_E = PC + Imm_Extension_E;
		end
	       else
		begin
		 ZeroE      = 1'b0; 
		 PCTarget_E = 32'b0;
		end
	      end
	BGE : begin
	       if(A >= B)
		begin
		 ZeroE      = 1'b1;
		 PCTarget_E = PC + Imm_Extension_E;
		end
	       else 
		begin
		 ZeroE      = 1'b0; 
		 PCTarget_E = 32'b0;
		end
	      end
	BLTU : begin
		if(a < b)
		 begin
		  ZeroE      = 1'b1;
		  PCTarget_E = PC + Imm_Extension_E;
		 end
		else
		 begin
		  ZeroE      = 1'b0; 
		  PCTarget_E = 32'b0; 
		 end
	       end					
	BGEU : begin
		if(a >= b)
		 begin
		  ZeroE      = 1'b1;
		  PCTarget_E = PC + Imm_Extension_E;
		 end
		else
		 begin
	  	  ZeroE      = 1'b0; 
		  PCTarget_E = 32'b0; 
		 end
	       end
	default : begin
		   Aluout     = 32'b0;
	 	   ZeroE      = 1'b0;
		   PCTarget_E = 32'b0;
		  end
      endcase
    end
   LUI:
    begin
     Aluout     = B;
     ZeroE      = 1'b0;
     PCTarget_E = 32'b0;				
    end
   AUIPC:
    begin
     Aluout     = PC+B;
     ZeroE      = 1'b0;
     PCTarget_E = 32'b0;
    end
   JAL:
    begin
     Aluout     = PC+32'h00000004;
     PCTarget_E = PC+B;
     ZeroE      = 1'b0;
    end
   JALR:
    begin
     Aluout     = PC+32'h00000004;
     PCTarget_E = ((PC+B)&32'hFFFFFFFE);
     ZeroE      = 1'b0;
    end
/* CSR:
    begin
     case(Instruction [14:12])
      CSRRW || CSRRS || CSRRC || CSRRWI || CSRRSI || CSRRCI:
	begin
	 Aluout <= {{20{1'b0}},Instruction [20:31]};
	 PCTarget_E <= PCTarget_E;
	end
      endcase
     end
*/
   default : begin
    	      Aluout     = 32'b0;
   	      PCTarget_E = 32'b0;
   	      ZeroE      = 1'b0;
   	     end
  endcase
 end
endmodule 
/*
module Alu (Instruction, Imm_Extension_E, Aluout, A,B, ZeroE,PC, PCTarget_E);
input [31:0] Instruction;
input signed [31:0] Imm_Extension_E;
input signed [31:0] A,B;
input signed [31:0] PC;
output reg [31:0] Aluout;
output reg ZeroE;  
output reg [31:0] PCTarget_E;

parameter [7:0] r_to_r   = 7'b0110011,
	  r_to_imm = 7'b0010011,
	  ld       = 7'b0000011,
	  store    = 7'b0100011,
	  branch   = 7'b1100011;

// REGISTER TO REGISTER TYPE
parameter [9:0] add = 10'b0000000_000,
		sub = 10'b0100000_000,
		slt = 10'b0000000_010,
		sll = 10'b0000000_001,
		sltu= 10'b0000000_011,
		Xor = 10'b0000000_100,
		srl = 10'b0000000_101,
		sra = 10'b0100000_101,
		Or  = 10'b0000000_110,
		And = 10'b0000000_111;

// REGISTER TO IMMEDIATE TYPE
parameter [2:0] addi = 3'b000,
	        slli = 3'b001,
                slti = 3'b010,
		sltui= 3'b011,
		Xori = 3'b100,
		srli = 3'b101,
		//srai = 3'b101,
		Ori  = 3'b110,
		andi = 3'b111;

//LOAD TYPE
parameter [2:0] LB = 3'b000,
		LH = 3'b001,
		LW = 3'b010,
		LBU= 3'b100,
		LHU= 3'B101;

//STORE TYPE

parameter [2:0] SB = 3'b000, // STORE BYTE 
		SH = 3'b001, // STORE HALF WORD
		SW = 3'b010; // STORE WORD

// UPPER IMMEDIATE TYPE
parameter [6:0] LUI = 7'b0110111, // LOAD UPPER IMMEDIATE 
		AUIPC =7'b0010111; //ADD UPPER IMMEDIATE 
// BRANCH TYPE
parameter [2:0] BEQ = 3'b000, // BRANCH EQUAL
	     	BNE = 3'b001, // BRANCH NOT EQUAL
		BLT = 3'b100, // BRANCH LESS THAN
		BGE = 3'b101, // BRANCH GREATER THAN
		BLTU= 3'b110, // BRANCH LESS THAN UNSINGNED 
		BGEU= 3'b111; // BRANCH GREATER THAN UNSINGED
//JUMP TYPE
parameter [6:0] JAL = 7'b1101111,
		JALR = 7'b1100111; 

always@(*)
begin
	case(Instruction [6:0] )
		r_to_r :
		begin
			//ZeroE  = 0;
			//PCTarget_E = PCTarget_E;
			case({Instruction [31:25] ,Instruction[14:12]})
			
				add : Aluout = A + B;
				sub : Aluout = A - B;
				sll : Aluout = A << B[4:0];
				slt : 
				begin
					if( A < B)
						Aluout <= 1;
					else
						Aluout <= 0; 
				end
				sltu: 
				begin
					if($unsigned(A) < $unsigned(B))
						Aluout = 1;
					else
						Aluout = 0; 
				end
				Xor : Aluout = A ^ B;
				srl : Aluout = A >> B[4:0];
				sra : Aluout = A >>> B[4:0];
				Or  : Aluout = A || B;
				And : Aluout = A && B;
				        
			default :begin
				  Aluout = 32'b0;
				  ZeroE = 1'b0;
				  PCTarget_E = PCTarget_E;
				 end 
			endcase 
		end
		
		r_to_imm:
		begin
			//ZeroE  = 0;
			//PCTarget_E = PCTarget_E;
			  case(Instruction[14:12] )
				addi : Aluout = A + B;
				slli : Aluout = A << B;
				slti :  
				begin
					if(A < B)
						Aluout = 1;
					else
						Aluout = 0; 
				end
				sltui: 
				begin
					if($unsigned(A) < $unsigned(B))
						Aluout = 1;
					else
						Aluout = 0; 
				end
				Xori : Aluout = A ^ B;
				srli :
				begin
					if(B[10]  == 0)
						Aluout = A >> B;
				end	
				srai : 
				begin
					if(B[10]  == 1)
						Aluout = A >>> B;
				end
				Ori : Aluout = A | B;
				andi : Aluout = A & B;
				        
			default :begin
				  Aluout = 32'b0;
				  ZeroE = 1'b0;
				  PCTarget_E = PCTarget_E;
				 end
			endcase
		end
			
			ld:
			begin
				//ZeroE  = 0;
			//PCTarget_E = PCTarget_E;
				case(Instruction [14:12])
					LB : Aluout = A + B;
					LH : Aluout = A + B;
					LW : Aluout = A + B;
					LBU : Aluout = $unsigned(A) + $unsigned(B);
					LHU : Aluout = $unsigned(A) + $unsigned(B);
				default :begin
				  Aluout = 32'b0;
				  ZeroE = 1'b0;
				  PCTarget_E = PCTarget_E;
				 end
				endcase
			end
			
			store:
			begin
				//ZeroE = 1'b0;
				  //PCTarget_E = PCTarget_E;
				case(Instruction [14:12])
					SB : Aluout = A +B;
					SH : Aluout = A + B;
					SW : Aluout = A + B;
					default :begin
				  Aluout = 32'b0;
				  ZeroE = 1'b0;
				  PCTarget_E = PCTarget_E;
				 end
				endcase
			end
			
			branch:
			begin
				//Aluout = 32'b0;
				case(Instruction [14:12])
					BEQ :
					begin
						
						if(A == B)begin
							ZeroE = 1;
							PCTarget_E = PC + Imm_Extension_E;end
						else begin
							ZeroE = 0; 
							PCTarget_E = PCTarget_E; end
					end
					BNE :
					begin
						if(A != B)begin
							ZeroE = 1;
							PCTarget_E = PC + Imm_Extension_E ;end
						
						else begin
							ZeroE = 0; 
							PCTarget_E = PCTarget_E; end
					end
					BLT :
					begin
						if(A < B)begin
							ZeroE = 1;
							PCTarget_E = PC + Imm_Extension_E; end
						
						else begin
							ZeroE = 0; 
							PCTarget_E = PCTarget_E; end

					end
					BGE :
					begin
						if(A >= B)begin
							ZeroE = 1;
							PCTarget_E = PC + Imm_Extension_E;end
			
						else begin
							ZeroE = 0; 
							PCTarget_E = PCTarget_E; end
						
					end
					BLTU :
					begin
						if($unsigned(A) < $unsigned(B))begin
							ZeroE = 1;
							PCTarget_E = PC + Imm_Extension_E;
							end
						else begin
							ZeroE = 0; 
							PCTarget_E = PCTarget_E; end

					end					
					BGEU :
					begin
						if($unsigned(A) >= $unsigned(B))
						begin
							ZeroE = 1;
							PCTarget_E = PC + Imm_Extension_E;
						end
						else begin
							ZeroE = 0; 
							PCTarget_E = PCTarget_E; end
						
					end
					default :begin
  				  		  Aluout = 32'b0;
						  ZeroE = 1'b0;
						  PCTarget_E = PCTarget_E;
				 end
				endcase
			end
			LUI:
			begin
				Aluout = B;				
			end
			AUIPC:
			begin
				Aluout = PC+B;
			end
			JAL:
			begin
				Aluout = PC+32'h00000004;
				PCTarget_E = PC+B;
			end
			JALR:
			begin
				Aluout = PC+32'h00000004;
				PCTarget_E = ((PC+B)&32'hFFFFFFFE);
			end
	default:begin
	        ZeroE  = ZeroE;
		PCTarget_E = PCTarget_E;
	        end
	endcase
end
endmodule 
*/