module Decode_cycle(clk, rst, clear, RegWrite_W, RD_W, Instruction_D, PC_D, Result_W, RegWrite_E, MemWrite_E, 
Result_Select_E, Branch_E, Jump_E, Rs2_Select_E, RD1_E, RD2_E, Imm_Extension_E, RS1_E, RS2_E, RD_E, PC_E, Instruction_E);

   // Declaring Inputs and Outputs
    input clk, rst, clear, RegWrite_W;
    input [4:0] RD_W;
    input [31:0] Instruction_D, PC_D, Result_W;

    output RegWrite_E, MemWrite_E, Result_Select_E, Branch_E, Jump_E, Rs2_Select_E;
    output signed [31:0] RD1_E, RD2_E, Imm_Extension_E;
    output [4:0] RS1_E, RS2_E, RD_E;
    output [31:0] PC_E,Instruction_E;

   // Declare Interim Wires
    wire RegWrite_D, MemWrite_D, Result_Select_D, Branch_D, Jump_D, Rs2_Select_D;
    wire signed [31:0] RD1_D, RD2_D, Imm_Extension_D;
    wire [31:0] Instruction_D;

   // Declaration of Interim Register
    reg RegWrite_D_reg, MemWrite_D_reg, Result_Select_D_reg, Branch_D_reg, Jump_D_reg, Rs2_Select_D_reg;
    reg [31:0] RD1_D_reg, RD2_D_reg, Imm_Extension_D_reg;
    reg [4:0] RS1_D_reg, RS2_D_reg, RD_D_reg;
    reg [31:0] PC_D_reg, Instruction_D_reg;


   // Initiate the modules
    // Control Unit
    Control_Unit Control_Unit(.Opcode(Instruction_D[6:0]),
                              .RegWrite(RegWrite_D),
                              .MemWrite(MemWrite_D),
                              .Result_Select(Result_Select_D),
                              .Branch(Branch_D),
  			      .Jump(Jump_D),
			      .Rs2_Select(Rs2_Select_D)
                             );

    // Register File
    Register_File Registers(.clk(clk),
                            .rst(rst),
                            .WE3(RegWrite_W),
                            .WD3(Result_W),
                            .A1(Instruction_D[19:15]),
                            .A2(Instruction_D[24:20]),
                            .A3(RD_W),
                            .RD1(RD1_D),
                            .RD2(RD2_D)
                           );

    // Sign Extension
    Sign_Extension Extension (
                        .Instruction(Instruction_D[31:0]),
                        .Imm(Imm_Extension_D)
                        );

    // Declaring Register Logic
    always @(posedge clk or posedge rst)
     begin
      if(rst == 1'b1) 
       begin
        RegWrite_D_reg <= 1'b0;
        MemWrite_D_reg <= 1'b0;
        Result_Select_D_reg <= 1'b0;
        Branch_D_reg <= 1'b0;
	Jump_D_reg <= 1'b0;
	Rs2_Select_D_reg <= 1'b0;
        RD1_D_reg <= 32'h00000000; 
        RD2_D_reg <= 32'h00000000; 
        Imm_Extension_D_reg <= 32'h00000000;
        RD_D_reg <= 5'h00;
        PC_D_reg <= 32'h00000000; 
        RS1_D_reg <= 5'h00;
        RS2_D_reg <= 5'h00;
        Instruction_D_reg <= 32'h00000000;
       end
      else if(clear)  //NOP
       begin
        RegWrite_D_reg <= 1'b0;
        MemWrite_D_reg <= 1'b0;
        Result_Select_D_reg <= 1'b0;
        Branch_D_reg <= 1'b0;
	Jump_D_reg <= 1'b0;
	Rs2_Select_D_reg <= 1'b0;
        RD1_D_reg <= 32'h00000000; 
        RD2_D_reg <= 32'h00000000; 
        Imm_Extension_D_reg <= 32'h00000000;
        RD_D_reg <= 5'h00;
        PC_D_reg <= 32'hxxxxxxxx; 
        RS1_D_reg <= 5'h00;
        RS2_D_reg <= 5'h00;
        Instruction_D_reg <= 32'h00000033;
       end
      else
       begin
        RegWrite_D_reg <= RegWrite_D;
        MemWrite_D_reg <= MemWrite_D;
        Result_Select_D_reg <= Result_Select_D;
        Branch_D_reg <= Branch_D;
	Jump_D_reg <= Jump_D;
	Rs2_Select_D_reg <= Rs2_Select_D;
        RD1_D_reg <= RD1_D;
        RD2_D_reg <= RD2_D; 
        Imm_Extension_D_reg <= Imm_Extension_D;
        RD_D_reg <= Instruction_D[11:7];
        PC_D_reg <= PC_D;
        RS1_D_reg <= Instruction_D[19:15];
        RS2_D_reg <= Instruction_D[24:20];
        Instruction_D_reg <= Instruction_D;
       end
     end

    // Output asssign statements
    assign RegWrite_E = RegWrite_D_reg;
    assign MemWrite_E = MemWrite_D_reg;
    assign Result_Select_E = Result_Select_D_reg;
    assign Branch_E = Branch_D_reg;
    assign Jump_E = Jump_D_reg;
    assign Rs2_Select_E = Rs2_Select_D_reg;
    assign RD1_E = RD1_D_reg;
    assign RD2_E = RD2_D_reg;
    assign Imm_Extension_E = Imm_Extension_D_reg;
    assign RD_E = RD_D_reg;
    assign PC_E = PC_D_reg;
    assign RS1_E = RS1_D_reg;
    assign RS2_E = RS2_D_reg;
    assign Instruction_E = Instruction_D_reg;
endmodule 