module Execute_cycle(clk, rst, RegWrite_E, MemWrite_E, Result_Select_E, Branch_E, Jump_E, Rs2_Select_E, 
RD1_E, RD2_E, Imm_Extension_E, Instruction_E, RD_E, PC_E, Result_W, ForwardA_E, ForwardB_E, PC_Select_E, 
RegWrite_M, MemWrite_M, Result_Select_M, RD_M, WriteData_M, Alu_Result_M, PCTarget_E, Operation_M);

    // Declaration of Inputs and Outputs
    input clk, rst, RegWrite_E,MemWrite_E,Result_Select_E,Branch_E,Jump_E,Rs2_Select_E;
    input [31:0] RD1_E, RD2_E, Imm_Extension_E;
    input [31:0] Instruction_E;
    input [4:0] RD_E;
    input [31:0] PC_E;
    input [31:0] Result_W;
    input [1:0] ForwardA_E, ForwardB_E;

    output PC_Select_E, RegWrite_M, MemWrite_M, Result_Select_M;
    output [4:0] RD_M; 
    output [31:0] WriteData_M; 
    output [31:0] Alu_Result_M;
    output [31:0] PCTarget_E;
    output [2:0] Operation_M;

    // Declaration of Interim Wires
    wire [31:0] A, B_temp;
    wire [31:0] B;
    wire [31:0] Alu_Result_E;
    wire ZeroE;
    wire [31:0] Store_extension;
    
    // Declaration of Register
    reg RegWrite_E_reg, MemWrite_E_reg, Result_Select_E_reg;
    reg [4:0] RD_E_reg;
    reg [31:0] WriteData_E_reg; 
    reg [31:0] Alu_Result_E_reg;
    reg [2:0] Operation_E_reg;

    // Declaration of Modules
    // 3 by 1 Mux for A
    mux3_1 mux_A(.a(RD1_E),
		 .b(Alu_Result_M),
                 .c(Result_W),
                 .s(ForwardA_E),
                 .y(A)
                );

    // 3 by 1 Mux for B_temp
    mux3_1 mux_B_temp(.a(RD2_E),
		      .b(Alu_Result_M),
                      .c(Result_W),
                      .s(ForwardB_E),
                      .y(B_temp)
                     );
    // Mux for B
    mux2_1 mux_B(.a(B_temp),
                 .b(Imm_Extension_E),
                 .s(Rs2_Select_E),
                 .y(B)
                );

    // ALU Unit
  Alu alu(.Instruction(Instruction_E),
	  .Imm_Extension_E(Imm_Extension_E),
          .Aluout(Alu_Result_E),
          .a(A),
          .b(B),
          .ZeroE(ZeroE),
          .PC(PC_E),
          .PCTarget_E(PCTarget_E)
         );

 Store_Extension Store(.WriteData_E_temp(B_temp),
		       .Operation(Instruction_E[14:12]),
		       .WriteData_E(Store_extension)
                      );

    // Register Logic
    always @(posedge clk or posedge rst)
     begin
      if(rst == 1'b1)
       begin
        RegWrite_E_reg <= 1'b0; 
        MemWrite_E_reg <= 1'b0; 
        Result_Select_E_reg <= 1'b0;
        RD_E_reg <= 5'h00;
        WriteData_E_reg <= 32'h00000000; 
        Alu_Result_E_reg <= 32'h00000000;
        Operation_E_reg <= 3'b000;
       end
      else
       begin
        RegWrite_E_reg <= RegWrite_E; 
        MemWrite_E_reg <= MemWrite_E; 
        Result_Select_E_reg <= Result_Select_E;
        RD_E_reg <= RD_E;
        WriteData_E_reg <= Store_extension; 
        Alu_Result_E_reg <= Alu_Result_E;
        Operation_E_reg <= Instruction_E[14:12];
       end
     end

    // Output Assignments
    assign PC_Select_E = Jump_E|(ZeroE & Branch_E);
    assign RegWrite_M = RegWrite_E_reg;
    assign MemWrite_M = MemWrite_E_reg;
    assign Result_Select_M = Result_Select_E_reg;
    assign RD_M = RD_E_reg;
    assign WriteData_M = WriteData_E_reg;
    assign Alu_Result_M = Alu_Result_E_reg;
    assign Operation_M = Operation_E_reg;

endmodule 