module Memory_cycle(clk, rst, RegWrite_M, MemWrite_M, Result_Select_M, RD_M, WriteData_M, 
    Alu_Result_M, Operation_M, RegWrite_W, Result_Select_W, RD_W, Alu_Result_W, ReadData_W);
    
    // Declaration of I/Os
    input clk, rst, RegWrite_M, MemWrite_M, Result_Select_M;
    input [4:0] RD_M; 
    input [31:0] WriteData_M, Alu_Result_M;
    input [2:0] Operation_M;

    output RegWrite_W, Result_Select_W; 
    output [4:0] RD_W;
    output [31:0] Alu_Result_W, ReadData_W;

    // Declaration of Intermediate Wires
    wire [31:0] ReadData_M,ReadData_M_temp ;

    // Declaration of Intermediate Registers
    reg RegWrite_M_reg, Result_Select_M_reg;
    reg [4:0] RD_M_reg;
    reg [31:0] Alu_Result_M_reg, ReadData_M_reg;

    // Declaration of Module Initiation
    Data_Memory Data_Memory(.clk(clk),
                            .rst(rst),
                            .WE(MemWrite_M),
                            .WD(WriteData_M),
                            .A(Alu_Result_M),
                            .RD(ReadData_M_temp)
                           ); 
    Load_Extension Load(.ReadData_M_temp(ReadData_M_temp),
                        .Operation(Operation_M),
                        .ReadData_M(ReadData_M)
		       );
    // Memory Stage Register Logic
    always @(posedge clk or posedge rst)
     begin
      if (rst == 1'b1)
       begin
        RegWrite_M_reg <= 1'b0; 
        Result_Select_M_reg <= 1'b0;
        RD_M_reg <= 5'b00000;
        Alu_Result_M_reg <= 32'h00000000; 
        ReadData_M_reg <= 32'h00000000;
       end
      else
       begin
        RegWrite_M_reg <= RegWrite_M; 
        Result_Select_M_reg <= Result_Select_M;
        RD_M_reg <= RD_M;
        Alu_Result_M_reg <= Alu_Result_M; 
        ReadData_M_reg <= ReadData_M;
       end
    end 

    // Declaration of output assignments
    assign RegWrite_W = RegWrite_M_reg;
    assign Result_Select_W = Result_Select_M_reg;
    assign RD_W = RD_M_reg;
    assign Alu_Result_W = Alu_Result_M_reg;
    assign ReadData_W = ReadData_M_reg;

endmodule 