/*
`include "Fetch_cycle.v"
`include "Decode_cycle.v"
`include "Execute_cycle.v"
`include "Memory_cycle.v"
`include "Writeback_cycle.v"
`include "Hazardunit.v"    
`include "Program_Counter.v"
`include "PC_Adder.v"
`include "mux2_1.v"
`include "mux3_1.v"
`include "Instruction_Memory.v"
`include "Control_Unit.v"
`include "Register_File.v"
`include "Sign_Extension.v"
`include "Alu.v"
`include "Data_Memory.v"
`include "Load_Extension.v"
`include "Store_Extension.v"
*/

module Pipeline(clk, rst,PC_D,Imm_Extension_E,RD1_E,RD2_E,Instruction_D,ReadData_W, Result_W);

    // Declaration of Inputs
    input clk, rst;
    input [31:0]PC_D,Imm_Extension_E;//Instruction_enable;
    output [31:0] Instruction_D,ReadData_W,Result_W;
   // input [31:0] instruction_fetch;
wire enable, clear, clear_d;
wire [31:0] PC_E;
wire [31:0]Instruction_E;
wire [4:0]RS1_E,RS2_E,RD_E,RD_M,RD_W;
input [31:0]RD1_E,RD2_E;
wire RegWrite_E,RegWrite_M,RegWrite_W;
wire MemWrite_E,MemWrite_M;
wire Result_Select_E,Result_Select_M,Result_Select_W;
wire Branch_E, Jump_E, Rs2_Select_E, PC_Select_E;
wire [31:0]PCTarget_E;
wire [31:0]Alu_Result_M,Alu_Result_W;
wire [31:0]WriteData_M;
wire [1:0] ForwardA_E, ForwardB_E;
wire [2:0] Operation_M;
    // Module Initiation
    // Fetch Stage
    Fetch_cycle Fetch(.clk(clk), 
                      .rst(rst),
                      .enable(enable),
                      .clear_d(clear_d), 
                      .PC_Select_E(PC_Select_E), 
                      .PCTarget_E(PCTarget_E), 
                      .Instruction_D(Instruction_D), 
                      .PC_D(PC_D)
                   //   .instruction_fetch(instruction_fetch),
                   //   .Instruction_enable(Instruction_enable)
                     );

    // Decode Stage
    Decode_cycle Decode(.clk(clk), 
                        .rst(rst),
                        .clear(clear),
                        .RegWrite_W(RegWrite_W), 
                        .RD_W(RD_W), 
                        .Instruction_D(Instruction_D), 
                        .PC_D(PC_D),     
                        .Result_W(Result_W), 
                        .RegWrite_E(RegWrite_E),  
                        .MemWrite_E(MemWrite_E), 
                        .Result_Select_E(Result_Select_E),
                        .Branch_E(Branch_E),
			.Jump_E(Jump_E),
                        .Rs2_Select_E(Rs2_Select_E),   
                        .RD1_E(RD1_E), 
                        .RD2_E(RD2_E), 
                        .Imm_Extension_E(Imm_Extension_E),
                        .RS1_E(RS1_E),
                        .RS2_E(RS2_E), 
                        .RD_E(RD_E), 
                        .PC_E(PC_E),
                        .Instruction_E(Instruction_E)
                       ); 
 
    // Execute Stage
    Execute_cycle Execute(.clk(clk), 
                          .rst(rst), 
                          .RegWrite_E(RegWrite_E),  
                          .MemWrite_E(MemWrite_E), 
                          .Result_Select_E(Result_Select_E), 
                          .Branch_E(Branch_E),
			  .Jump_E(Jump_E),
			  .Rs2_Select_E(Rs2_Select_E), 
                          .RD1_E(RD1_E), 
                          .RD2_E(RD2_E), 
                          .Imm_Extension_E(Imm_Extension_E),
			  .Instruction_E(Instruction_E), 
                          .RD_E(RD_E), 
                          .PC_E(PC_E),
                          .Result_W(Result_W),
                          .ForwardA_E(ForwardA_E),
                          .ForwardB_E(ForwardB_E), 
                          .PC_Select_E(PC_Select_E), 
                          .RegWrite_M(RegWrite_M),
                          .MemWrite_M(MemWrite_M), 
                          .Result_Select_M(Result_Select_M), 
                          .RD_M(RD_M), 
                          .WriteData_M(WriteData_M), 
                          .Alu_Result_M(Alu_Result_M),
                          .PCTarget_E(PCTarget_E),
			  .Operation_M(Operation_M)
                         );

    // Memory Stage
    Memory_cycle Memory(.clk(clk), 
                        .rst(rst), 
                        .RegWrite_M(RegWrite_M), 
                        .MemWrite_M(MemWrite_M), 
                        .Result_Select_M(Result_Select_M), 
                        .RD_M(RD_M), 
                        .WriteData_M(WriteData_M), 
                        .Alu_Result_M(Alu_Result_M), 
                        .RegWrite_W(RegWrite_W), 
                        .Result_Select_W(Result_Select_W), 
                        .RD_W(RD_W),
                        .Alu_Result_W(Alu_Result_W), 
                        .ReadData_W(ReadData_W),
			.Operation_M(Operation_M)
                       );

    // Write Back Stage
    Writeback_cycle WriteBack(
                              .Result_Select_W(Result_Select_W), 
                              .Alu_Result_W(Alu_Result_W), 
                              .ReadData_W(ReadData_W), 
                              .Result_W(Result_W)
                             );
 
    // Hazard Detection Unit
    Hazardunit  Hazard_Detection_Unit(.clear(clear),
                                      .clear_d(clear_d),
                                      .enable(enable),
                                      .ForwardA_E(ForwardA_E),
                                      .ForwardB_E(ForwardB_E),
                                      .opcode_D(Instruction_D[6:0]),
				      .opcode_E(Instruction_E[6:0]),
                                      .RS1_E(RS1_E),
                                      .RS2_E(RS2_E),
                                      .RD_E(RD_E),
                                      .RD_M(RD_M),
                                      .RD_W(RD_W),
                                      .RS1_D(Instruction_D[19:15]),
                                      .RS2_D(Instruction_D[24:20]),
                                      .RegWrite_M(RegWrite_M),
                                      .RegWrite_W(RegWrite_W),
                                      .Result_Select_E(Result_Select_E),
                                      .PC_Select_E(PC_Select_E)
                                     );

endmodule 